#ifndef CRYSTALDATA_H
#define CRYSTALDATA_H

#include <QVector>
#include <QStringList>

namespace crydata {

const int SIZELAB = 17;

enum targetType {TARG_BOND_TABLE=1, TARG_CURRENT, TARG_MOL_MEC};

typedef struct {
    QString label;
    float xyz[3];
    float biso;
    float occ;
    int rcod[5];
    int doc;
    int kscatt;
} AtomProperties;

typedef struct {
    char lab[SIZELAB];
    float xyz[3];
    float biso;
    float occ;
    int rcod[5];
    int doc;
    int kscatt;
} c_atom_type;

typedef struct {
    float scal;
    int scalcod;
} crystal_t;

typedef struct {
    int code;
    QVector<int> atm;
    QStringList type;
    float val, targ, sigma;
    bool active;
} RestraintProperties;

typedef struct {
    int code;
    QVector<int> atm;
    QVector<int> par;
} ConstraintProperties;

typedef struct {
    QString lab;
} scatter_type;

QString crystalPhaseName(int idph);
QVector<scatter_type> make_scatters(int phId);
void get_restraints_settings(int phId, QVector<double> &wei, int &targetType);
void make_restraints(int phId, QVector<crydata::RestraintProperties> &restr, int kaction);
void make_constraints(int phId, QVector<crydata::ConstraintProperties> &constr);
void put_restraints(int phId, const QVector<crydata::RestraintProperties> &restr, const QVector<double> &wei);
int isParInConstraint(QVector<ConstraintProperties> &constr, int atom, int par);
int getAtomCode(const AtomProperties &atom);

}
#endif // CRYSTALDATA_H
