#ifndef DOFWIDGET_H
#define DOFWIDGET_H

#include "checkboxheader.h"

#include <QWidget>
#include <QStandardItemModel>
#include <QSortFilterProxyModel>

typedef struct {
    int id;
    QString label;
    float biso;
    float occ;
    int bisoCode;
    int doc;
} AtomInfo;

typedef struct {    
    int atomIndex[4];
    int rcod;
    float torsion;
    float bound[2];
} RotationInfo;

typedef struct {
    int nat;
    QVector<int> atomIndex;
    int rcod[6];
    float tlow[3];
    float thigh[3];
    float rbound[2];
    int rat;
} MoleculeInfo;

namespace Ui {
class DOFWidget;
}

class DOFWidget : public QWidget
{
    Q_OBJECT

public:
    explicit DOFWidget(QWidget *parent = nullptr);
    ~DOFWidget();
    void setWidget(int id);
    int getNumberParam() const;
    void updateComputingVariables();

private slots:
    void on_atomsPushButton_clicked();
    void on_docPushButton_clicked();
    void intDofSelected(const QModelIndex &index);
    void extDofHeaderCheckBoxClicked(bool state);
    void extDofRowCheckBoxClicked(const QModelIndex &index, bool state);
    void intDofHeaderCheckBoxClicked(bool state);
    void intDofRowCheckBoxClicked(const QModelIndex &index, bool state);
    void fragmentChanged(QStandardItem *item);
    void intDofChanged(QStandardItem *item);
    void on_comRadioButton_clicked();
    void on_atomRadioButton_clicked();

signals:
    void parameterChecked(int par);

private:
    Ui::DOFWidget *ui;
    int phaseId;
    int selectedFragment;
    int numberParam;
    int hklPO[3];
    int codePO;
    float parPO;
    bool parTableChangeEnabled;
    bool dofTableChangeEnabled;
    QVector<AtomInfo> atoms;
    QVector<MoleculeInfo> fragments;
    QVector<RotationInfo> rotations;
    QStandardItemModel *fragmentsModel;
    QSortFilterProxyModel *fragmentsProxyModel;
    QStandardItemModel *parametersModel;
    QStandardItemModel *intDOFModel;
    QSortFilterProxyModel *intDOFProxyModel;
    CheckBoxHeader *extDofCbHeader;
    CheckBoxHeader *intDofCbHeader;

    void makeFragmentsAndRotations();
    void getPatternInfo();
    QString getFragmentLabel(int index);
    QString getRotationLabel(int index);
    void createFragmentsModel();
    void updateFragmentModel();
    void setFragmentTableView();
    void createParametersModel();
    void setParametersTableView();
    void updateParametersModel(int index);
    void createIntDOFModel();
    void setIntDOFTableView();
    void updateIntDOFModel();
    int rowCheckBoxClickedCount(const QStandardItemModel *model, int col) const;    

    enum parTableCol {ATOM_COL, REF_COL, MIN_COL, MAX_COL};
    enum dofTableCol {TORSION_COL, DOFREF_COL, VALUE_COL, DOFMIN_COL, DOFMAX_COL};
};

#endif // DOFWIDGET_H
