#include "exporttabledialog.h"
#include "ui_exporttabledialog.h"

#include <QCheckBox>
#include <QSignalMapper>

ExportTableDialog::ExportTableDialog(const QStringList &columnTitle, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ExportTableDialog)
{
    ui->setupUi(this);

    initDialog(columnTitle);
}

ExportTableDialog::ExportTableDialog(const QTableView *table, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ExportTableDialog)
{
    ui->setupUi(this);

    QStringList columnTitle;
    for(int c = 0; c < table->model()->columnCount(); c++) {
        if (!table->isColumnHidden(c))
            columnTitle << table->model()->headerData(c,Qt::Horizontal).toString();
    }

    initDialog(columnTitle);
}

ExportTableDialog::~ExportTableDialog()
{
    delete ui;
}

void ExportTableDialog::initDialog(const QStringList &columnTitle)
{
    ui->label->setStyleSheet("QLabel{color: green;}");

    auto signalMapper = new QSignalMapper(this);
#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
    connect(signalMapper, QOverload<int>::of(&QSignalMapper::mapped), this, &ExportTableDialog::checkClicked);
#else
    connect(signalMapper, QOverload<int>::of(&QSignalMapper::mappedInt), this, &ExportTableDialog::checkClicked);
#endif
    checkState.fill(Qt::Checked, columnTitle.size());

    for (int i = 0; i < columnTitle.size(); i++) {
        auto checkBox = new QCheckBox(columnTitle.at(i), this);
        checkBox->setChecked(checkState.at(i));
        ui->horizontalLayout->addWidget(checkBox);
        signalMapper->setMapping(checkBox,i);
        connect(checkBox, SIGNAL(clicked()), signalMapper, SLOT(map()));
        //connect(checkBox, &QCheckBox::clicked, signalMapper, &QSignalMapper::map);
    }
}

void ExportTableDialog::checkClicked(int checkId)
{
    checkState[checkId] = checkState[checkId]? Qt::Unchecked : Qt::Checked;
}

QVector<int> ExportTableDialog::getCheckState() const
{
    return checkState;
}
