#include "formulawidget.h"
#include "ui_formulawidget.h"

FormulaWidget::FormulaWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::FormulaWidget)
{
    ui->setupUi(this);

    //deprecated in Qt6 int width = ui->volumeLabel->fontMetrics().width(".1234567");
    int width = QFontMetrics(ui->volumeLabel->font()).horizontalAdvance(".1234567");
    ui->volumeLabel->setMinimumWidth(width);
    ui->volumePerAtomLabel->setMinimumWidth(width);
    ui->densityLabel->setMinimumWidth(width);
    ui->mwLabel->setMinimumWidth(width);

    ui->formulaLineEdit->setInfoEnabled(true);
    connect(ui->formulaLineEdit, &FormulaEditor::formulaChanged, this, [=](){
        updateInfoLineEdit();
        emit formulaEdited(ui->formulaLineEdit->isValid());
    });
}

void FormulaWidget::updateInfoLineEdit()
{
    if (ui->formulaLineEdit->isValid()) {
        ui->volumeLabel->setText(QString::number(ui->formulaLineEdit->volume()));
        if (ui->formulaLineEdit->volumePerAtom() > 0)
            ui->volumePerAtomLabel->setText(QString::number(ui->formulaLineEdit->volumePerAtom()));
        else
            ui->volumePerAtomLabel->clear();
        if (ui->formulaLineEdit->density() > 0)
            ui->densityLabel->setText(QString::number(ui->formulaLineEdit->density()));
        else
            ui->densityLabel->clear();
        ui->mwLabel->setText(QString::number(ui->formulaLineEdit->MW()));
    } else {
        ui->volumeLabel->clear();
        ui->volumePerAtomLabel->clear();
        ui->densityLabel->clear();
        ui->mwLabel->clear();
    }
}

FormulaWidget::~FormulaWidget()
{
    delete ui;
}

void FormulaWidget::updateFormula(const QString &formula, double volume)
{
    ui->formulaLineEdit->setCellVolume(volume);
    ui->formulaLineEdit->setText(formula);
}

void FormulaWidget::updateInfo(double volume)
{
    ui->formulaLineEdit->setCellVolume(volume);
    ui->formulaLineEdit->validateFormula();
    updateInfoLineEdit();
}

QString FormulaWidget::formula() const
{
    return ui->formulaLineEdit->text();
}

bool FormulaWidget::isValidFormula() const
{
    return ui->formulaLineEdit->isValid();
}
