#include "fourierpeaklistdialog.h"
#include "ui_fourierpeaklistdialog.h"
#include "floatdelegate.h"
#if defined(JAV_WIDGET)
#include "crystalviewwidget.h"
#include "mainwindow.h"
#else
#include "javwindow.h"
#endif

#include <QDebug>

FourierPeakListDialog::FourierPeakListDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::FourierPeakListDialog),
    exitCode(0),
    nonFourierAtoms(0)
{
#if defined(JAV_WIDGET)
    //jav = qobject_cast<CrystalViewWidget*>(parent);
    MainWindow *mw = qobject_cast<MainWindow*>(parent);
    jav = mw->getJav();
#else
    jav = qobject_cast<JavWindow*>(parent);
#endif
    ui->setupUi(this);

    createFourPeakListModel();
    setFourPeakTableView();
}

FourierPeakListDialog::~FourierPeakListDialog()
{
    delete ui;
}

void FourierPeakListDialog::setDialog(FourierPeakInfo fPeaks[], int nPeaks, int nOldPeaks)
{
    fourPeakListModel->setRowCount(nPeaks);
    for (int r = 0; r < nPeaks; r++) {
        auto item = new QStandardItem(fPeaks[r].labPeak);
        fourPeakListModel->setItem(r,0,item);

        item = new QStandardItem();
        item->setData((int)round(fPeaks[r].intensity), Qt::DisplayRole);
        item->setTextAlignment(Qt::AlignRight);
        fourPeakListModel->setItem(r,1,item);

        item = new QStandardItem();
        item->setData(fPeaks[r].dist, Qt::DisplayRole);
        fourPeakListModel->setItem(r,2,item);

        item = new QStandardItem(fPeaks[r].labDist);
        fourPeakListModel->setItem(r,3,item);
    }

    ui->fourPeakTableView->resizeColumnsToContents();
    ui->fourPeakTableView->horizontalHeader()->setSectionResizeMode(3,QHeaderView::Stretch);

    nonFourierAtoms = nOldPeaks;
    exitCode = 0;
    selectedPeak.clear();
    ui->peakLabel->setText(QString("%1 new peaks found.").arg(nPeaks));
}

void FourierPeakListDialog::waitSignalIsEmitted()
{
    QEventLoop loop;
    connect(this,&QDialog::finished,&loop,&QEventLoop::quit);
    loop.exec();
}

void FourierPeakListDialog::on_buttonBox_clicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if(stdButton == QDialogButtonBox::Ok) {
        exitCode = 1;
    } else if (stdButton == QDialogButtonBox::YesToAll) {
        exitCode = 2;
    }else if (stdButton == QDialogButtonBox::Cancel) {
        exitCode = 3;
    }
}

void FourierPeakListDialog::markSelectedPeaks()
{
    QModelIndexList selection = ui->fourPeakTableView->selectionModel()->selectedRows();
    selectedPeak.resize(selection.count());
    for(int i=0; i < selection.count(); i++)
    {
        selectedPeak[i] = fourPeakListProxyModel->mapToSource(selection.at(i)).row() + nonFourierAtoms;
    }    
    if (jav->isVisible()) {
        jav->selectAtoms(selectedPeak);
    }
}

void FourierPeakListDialog::createFourPeakListModel()
{
    fourPeakListModel = new QStandardItemModel(this);
    fourPeakListProxyModel = new QSortFilterProxyModel(this);
    fourPeakListProxyModel->setSourceModel(fourPeakListModel);
    fourPeakListModel->setColumnCount(4);
    const QStringList titles = {"Label", "Intensity", "Minimum Distance", "From"};
    fourPeakListModel->setHorizontalHeaderLabels(titles);
}

void FourierPeakListDialog::setFourPeakTableView()
{
    ui->fourPeakTableView->setModel(fourPeakListProxyModel);
    ui->fourPeakTableView->verticalHeader()->setDefaultSectionSize(22);
    ui->fourPeakTableView->setSortingEnabled(true);
    ui->fourPeakTableView->sortByColumn(1,Qt::DescendingOrder);
    ui->fourPeakTableView->verticalHeader()->hide();
    ui->fourPeakTableView->setShowGrid(false);
    ui->fourPeakTableView->setAlternatingRowColors(true);
    ui->fourPeakTableView->setSelectionMode(QAbstractItemView::MultiSelection);
    ui->fourPeakTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    ui->fourPeakTableView->horizontalHeader()->stretchLastSection();
    ui->fourPeakTableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    ui->fourPeakTableView->setFocusPolicy(Qt::NoFocus);

    auto floatDelegate = new FloatDelegate(ui->fourPeakTableView);
    floatDelegate->setDecimals(3);
    ui->fourPeakTableView->setItemDelegateForColumn(2, floatDelegate);

    connect(ui->fourPeakTableView->selectionModel(), &QItemSelectionModel::selectionChanged,
            this, &FourierPeakListDialog::markSelectedPeaks);
}

int FourierPeakListDialog::getExitCode() const
{
    return exitCode;
}

void FourierPeakListDialog::getSelectedPeaks(FourierPeakInfo fPeaks[])
{
    if (exitCode == 1) {
        foreach(auto at, selectedPeak) {
            at -= nonFourierAtoms;
            fPeaks[at].selected = 1;
        }
    } else if (exitCode == 2) {
        for (int i = 0; i < fourPeakListModel->rowCount(); i++)
            fPeaks[i].selected = 1;
    } else {
        for (int i = 0; i < fourPeakListModel->rowCount(); i++)
            fPeaks[i].selected = 0;
    }
}
