#ifndef FOURIERPEAKLISTDIALOG_H
#define FOURIERPEAKLISTDIALOG_H

#include <QDialog>
#include <QAbstractButton>
#include <QStandardItemModel>
#include <QSortFilterProxyModel>

#if defined(JAV_WIDGET)
class CrystalViewWidget;
#else
class JavWindow;
#endif

typedef struct {
    char labPeak[17];
    float intensity;
    float  dist;
    char labDist[17];
    int selected;
} FourierPeakInfo;

namespace Ui {
class FourierPeakListDialog;
}

class FourierPeakListDialog : public QDialog
{
    Q_OBJECT

public:
    explicit FourierPeakListDialog(QWidget *parent = nullptr);
    ~FourierPeakListDialog();
    void setDialog(FourierPeakInfo fPeaks[], int nPeaks, int nOldPeaks);
    void waitSignalIsEmitted();
    int getExitCode() const;
    void getSelectedPeaks(FourierPeakInfo fPeaks[]);

private slots:
    void on_buttonBox_clicked(QAbstractButton *button);
    void markSelectedPeaks();

private:
    Ui::FourierPeakListDialog *ui;
    QStandardItemModel *fourPeakListModel;
    QSortFilterProxyModel *fourPeakListProxyModel;
    int exitCode, nonFourierAtoms;
    QVector<int> selectedPeak;
#if defined(JAV_WIDGET)
    CrystalViewWidget *jav;
#else
    JavWindow *jav;
#endif
    void createFourPeakListModel();
    void setFourPeakTableView();    
};

#endif // FOURIERPEAKLISTDIALOG_H
