#include "geocalc.h"
#include "ui_geocalc.h"
#if defined(JAV_WIDGET)
#include "crystalviewwidget.h"
#else
#include "javwindow.h"
#endif
#include "mainwindow.h"
#include <QFontDatabase>

GeoCalc::GeoCalc(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::GeoCalc)
{
#if defined(JAV_WIDGET)
    jav = qobject_cast<CrystalViewWidget*>(parent);
#else
    jav = qobject_cast<JavWindow*>(parent);
#endif
    ui->setupUi(this);
    ui->radioButton->hide();
}

GeoCalc::~GeoCalc()
{
    delete ui;
}

void GeoCalc::clear()
{
    ui->radioButton->setChecked(true);
    ui->labelHint->setText("");
    ui->results->clear();
}

void GeoCalc::on_radioDist_toggled(bool checked)
{
    if(!checked)
        return;
    jav->setMotionType(GEOM+2);
}

void GeoCalc::on_radioAngle_toggled(bool checked)
{
    if(!checked)
        return;
    jav->setMotionType(GEOM+3);
}

void GeoCalc::on_radioTors_toggled(bool checked)
{
    if(!checked)
        return;
    jav->setMotionType(GEOM+4);
}

void GeoCalc::setHint(QString msg)
{
    ui->labelHint->setTextFormat(Qt::RichText);
    ui->labelHint->setText("<b>"+msg+"</b>");
}

void GeoCalc::setRes(QString res)
{
    const QFont fixedFont = QFontDatabase::systemFont(QFontDatabase::FixedFont);
    ui->results->setFontFamily(fixedFont.family());
    ui->results->append(res);
}

void GeoCalc::reject()
{
    jav->setMotionType(NO_MOVE);
    QDialog::reject();
}
