#ifndef GLUTILS_H
#define GLUTILS_H
#include <QColor>
#include <QOpenGLWidget>
#include <QtGui/QOpenGLFunctions>
#include "libmath.h"
#include <vector>

typedef struct {
                 unsigned int type;
                 unsigned int model;
                 unsigned int object;
                 float zm;
               } PickResult;

typedef struct {
        float px, py, pz;      // Spline Control Coordinate
        float tx, ty, tz;      // Spline Direction Vector
        float hnx, hny, hnz;   // Horizontal Normal Vector
        float vnx, vny, vnz;   // Vertical Normal Vector
        float dx, dy, dz;      // Ribbon Height Vector
        float wx, wy, wz;      // Ribbon Width Vector
        float hinten;          // Horizontal Intensity
        float vinten;          // Vertical Intensity
        float hsize;           // Horizontal Vector Length
        float vsize;           // Vertical Vector Length
        float wide;            // Ribbon Width
    } Knot;

static const float no_mat[] = {0.0f, 0.0f, 0.0f, 1.0f};

void disegnaSfera (double radius, int slices, bool textureCoords=false);
void disegnaCilindro(double radius, double h, int slices);
void disegnaCono(double raggio, double h, int subdiv);
void render3DArrow(vet3 p1, vet3 p2, double raggio, int subdivisions);
bool FromCoordToWin(GLdouble objx, GLdouble objy, GLdouble objz,
              const GLfloat modelMatrix[16],
              const GLfloat projMatrix[16],
              const GLint viewport[4],
              GLdouble *winx, GLdouble *winy, GLdouble *winz);
bool FromWinToCoord(GLdouble winx, GLdouble winy, GLdouble winz,
              const GLfloat modelMatrix[16],
              const GLfloat projMatrix[16],
              const GLint viewport[4],
              GLdouble *objx, GLdouble *objy, GLdouble *objz);
vet3 DrawDistLines(int n, vet3 points[]);
vet3 DrawDistDihedral(vet3 p[]);
vet3 DrawDistAngles(int n, vet3 p[], double ang);
void Sfera(bool solid, double radius, int div);
QColor ToQColor(float c[],int n=3);
QColor ToQColor(float r, float g, float b, float alpha=1.0);
void FromQColor(QColor c, float color[], int n=4);
void renderSphere(float x, float y, float z, double raggio, int subdivisions,
                  bool iftexture = false);
void DrawCylinder(double v, double raggio, int subdivisions,
                      int d1=1, int d2=1, int ordleg=1);
void SetLightForSelected(QOpenGLFunctions *context, int b=0, float *mat_e = nullptr, float alpha=0.4f);
void SetLightSystem(QOpenGLFunctions *context, float colore[], float alpha, int blend);
void DrawPolygon(Vert v[]);
void renderEll(float x, float y, float z, double raggio, int subdivisions,
               float scala[], float mat[4][4]);
void ResetLightForSelected(QOpenGLFunctions *context);
void DrawSelection(QOpenGLFunctions *context);
int processHitsm (GLint hits, GLuint buffer[], std::vector<PickResult> &vresult);
void pickMatrix(double x, double y, double deltax, double deltay, GLint viewport[4]);
void renderCylinder(QOpenGLFunctions *f, vet3 p1, vet3 p2,
                    double raggio, float color1[] = nullptr,
                    int subdivisions=0, int d1=1, int d2=1);
void drawCylinder( vet3 p1, vet3 p2,
                   float raggio, float color1[]=nullptr);
void renderCircs(float x, float y, float z, double radius, int subdivisions,
               float scala[], float mat[4][4]);
void GetSizeandInten(Knot *knot);
#endif // GLUTILS_H
