#ifndef __GRAFICO_H
#define __GRAFICO_H

#include <QObject>
#include <QGraphicsScene>
#include <QGraphicsItem>
#include <QPainter>
#include <QStatusBar>
#include <QPushButton>
using namespace std;

namespace GraphicSpace 
{

typedef struct
               {
                double valmin;
                double valmax;
                int    bottom;
                int    left;
                int    right;
                int    top;
                int    deltax;
                int    deltay;
                double scale;
                double start;
                double step;
                int    tics;
                int    fprim;
                int    fsec;
                int    ncifre;
                char   ori;
                bool   ifSect;
                QString label;
               } Axes;

class Plot
{
public:
    Plot(QGraphicsScene *parent, int n, float x[], float y[]);
    void GetLim(float *x1, float *x2, float *y1, float *y2);
    void ToScreen();
    void Disegna();
    void DisegnaRiflessi(int bottom);
    bool IsRefl() {return ifRefl;}
    bool InLimits(QPoint p1);
    bool InLimits(int p1);
    static bool Cfrx(QPointF p1, QPointF p2) {return (p1.x() < p2.x()); }
    static bool Cfry(QPointF p1, QPointF p2) {return (p1.y() < p2.y()); }
    void setColor_line(const QColor &value);

private:
    int numPoints;
    vector<QPointF> points;
    vector<QPoint> ipoints;
    float xmin, ymin, xmax, ymax;
    bool ifLines, ifMarkers, ifRefl;
    QColor color_line, color_marker, color_refl;
    QGraphicsScene *m_Scene;
};

class Grafico   : public QGraphicsScene
{
public:
     Grafico(QStatusBar *bar, QPushButton *button, QObject *parent=0);
     void SetFunc(QString func1(QPointF)) {mMessage=func1;}
     void SetArea(int w, int h);
     void ClearArea(int x, int y, int w, int h);
     void ClearScene();
     void SetLim(float x1, float x2, float y1, float y2, bool force=false);
     QPoint ToScreen(QPointF p);
     QPointF FromScreen(QPoint p);
     void Aggiorna();
     void CalcolaScala();
     void AddPlot(float x[], float y[], int num, float tmin, float tmax,
                  bool use=true);
     void Disegna();
     void DrawAxes(Axes axe);
     void SetLabels(QString lx, QString ly) {labelx = lx; labely = ly;}
     void Restore();
     void DrawDashRect(int x, int y, int w, int h);
     void GetLim(QPointF *p1, QPointF *p2);
     void ResetLim();
     void SaveRestore (int action);
     void SetStartMove(bool val) {start_move=val;}
     void MouseRelease();
     void MouseMove(int x, int y);
     void ResetZoom();
     void DrawText(QFont f, QString s, QColor c, QPoint pos);
     void DrawVText(QFont f, QString s, QColor c, QPoint pos);
     void sizeChanged(int w, int h);
     int numPlots();
     void setColorPlot(int idPlot, const QColor& colorLine);

private:
    int Width, Height;
    float xmin0, xmax0, ymin0, ymax0;
    float xmin, xmax, ymin, ymax;
    float dx, dy, XScale, YScale;
    int top, right, bottom, left;
    int hfont, wfont;
    Axes assex, assey;
    vector <Plot> Plots;
    QString labelx, labely;
    QPoint labx, laby;
    QFont Gfont;
    QColor color_axes;
    bool start_move;
    int Xiniz;
    int Yiniz;
    int Wrect;
    int Hrect;
    QStatusBar *m_bar;
    QPushButton *resetButton;
    QString (*mMessage)(QPointF p);

protected:
    void mouseMoveEvent(QGraphicsSceneMouseEvent * mouseEvent) override;
    void mousePressEvent(QGraphicsSceneMouseEvent * mouseEvent) override;
    void mouseReleaseEvent(QGraphicsSceneMouseEvent * mouseEvent) override;
};


}               // end namespace

class AntialiasedLine : public QGraphicsItem {
public:
AntialiasedLine( const QPoint p1, const QPoint p2, const QColor c):
    P1(p1),
    P2(p2),
    mColor(c)
{
      Path.moveTo(P1);
      Path.lineTo(P2);
    //Path.quadTo( cX1 + ( cX2-cX1 )/2, cY1+30, cX2, cY2 );
    //scene->addItem( this );
}

~AntialiasedLine(){}

QRectF boundingRect() const {return QRectF( P1.x(), P1.y(), P2.x(), P2.y() );}

void paint(QPainter *painter, const QStyleOptionGraphicsItem *option=0, QWidget *widget=0){
        Q_UNUSED(option);
        Q_UNUSED(widget);
        painter->setPen( QPen(QBrush(mColor), 1, Qt::SolidLine, Qt::SquareCap, Qt::MiterJoin ) );
        painter->setRenderHint( QPainter::Antialiasing );
        painter->drawPath( Path );
}

private:
        QPainterPath Path;
        QPoint P1, P2;
        QColor mColor;
};

#endif
