#include "hbondsoptionsdialog.h"
#include "ui_hbondsoptionsdialog.h"
#include "properties.h"
#include <QDebug>
#include <QPushButton>
#include <QtMath>

extern "C" void gethboptions_(_ContactType donor[], _ContactType acceptor[], _ContactOptionType *HBopt);
extern "C" void puthboptions_(int *wm, const _ContactType donor[], const _ContactType acceptor[], const _ContactOptionType *HBopt);

HBondsOptionsDialog::HBondsOptionsDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::HBondsOptionsDialog)
{
    ui->setupUi(this);

    nDonors = ui->groupBoxDonor->children().count();
    nAcceptors = ui->groupBoxAcceptor->children().count();

    acceptor0 = new _ContactType[nAcceptors]();
    donor0 = new _ContactType[nDonors]();
    get_values_for_HBOpt(donor0,acceptor0,HBopt0);

    acceptorSave = new _ContactType[nAcceptors]();
    donorSave = new _ContactType[nDonors]();
}

HBondsOptionsDialog::~HBondsOptionsDialog()
{
    delete ui;
    delete [] donor0;
    delete [] acceptor0;
    delete [] acceptorSave;
    delete [] donorSave;
}

void HBondsOptionsDialog::on_buttonBox_clicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if(stdButton == QDialogButtonBox::Ok || stdButton == QDialogButtonBox::Apply)
    {
        _ContactOptionType HBopt;
        _ContactType *acceptor = new _ContactType[nAcceptors]();
        _ContactType *donor = new _ContactType[nDonors]();

        get_values_for_HBOpt(donor,acceptor,HBopt);
        int wm = Properties::GetActiveModel();
        puthboptions_(&wm, donor, acceptor, &HBopt);
    }
    else if (stdButton == QDialogButtonBox::RestoreDefaults)
    {
        set_values_for_HBOpt(donor0, acceptor0, HBopt0);
        int wm = Properties::GetActiveModel();
        puthboptions_(&wm, donor0, acceptor0, &HBopt0);
    }
}

void HBondsOptionsDialog::reject()
{
    RestoreHBoptions();
    QDialog::reject();
}

void HBondsOptionsDialog::get_values_for_HBOpt(_ContactType d[], _ContactType a[], _ContactOptionType &opt)
{
    QList<QCheckBox*> checkList = ui->groupBoxDonor->findChildren<QCheckBox*>();
    for (int i=0; i < checkList.count(); i++) {
        d[i].active = checkList.at(i)->checkState() == Qt::Checked;
        //d[i].z = donor0[i].z;  //Useless!
    }

    checkList = ui->groupBoxAcceptor->findChildren<QCheckBox*>();
    for (int i=0; i < checkList.count(); i++) {
        a[i].active = checkList.at(i)->checkState() == Qt::Checked;
        a[i].z = acceptor0[i].z;
    }

    opt.hpresent = ui->checkHPresent->checkState() ? 1 : 0;
    opt.tolmin = static_cast<float>(ui->spinTolMin->value());
    if (ui->comboMin->currentIndex() == 1) opt.tolmin = -opt.tolmin;
    opt.tolmax = static_cast<float>(ui->spinTolMax->value());
    if (ui->comboMax->currentIndex() == 1) opt.tolmax = -opt.tolmax;
    opt.intra = ui->checkBoxIntra->checkState() ? 1 : 0;
    if(opt.intra) opt.nminbond = ui->spinMinBond->value();
    opt.inter = ui->checkBoxInter->checkState() ? 1 : 0;
}

void HBondsOptionsDialog::set_values_for_HBOpt(const _ContactType d[], const _ContactType a[], const _ContactOptionType &opt)
{
    QList<QCheckBox*> checkList = ui->groupBoxDonor->findChildren<QCheckBox*>();
    for (int i=0; i < checkList.count(); i++) {
        checkList.at(i)->setCheckState(d[i].active ? Qt::Checked : Qt::Unchecked);
    }

    checkList = ui->groupBoxAcceptor->findChildren<QCheckBox*>();
    for (int i=0; i < checkList.count(); i++) {
        checkList.at(i)->setCheckState(a[i].active ? Qt::Checked : Qt::Unchecked);
    }

    ui->checkHPresent->setCheckState(opt.hpresent ? Qt::Checked : Qt::Unchecked);
    ui->spinTolMin->setValue(qFabs(opt.tolmin));
    ui->comboMin->setCurrentIndex(opt.tolmin >= 0 ? 0 : 1);
    ui->spinTolMax->setValue(qFabs(opt.tolmax));
    ui->comboMax->setCurrentIndex(opt.tolmax >= 0 ? 0 : 1);
    ui->checkBoxIntra->setCheckState(opt.intra ? Qt::Checked : Qt::Unchecked);
    ui->spinMinBond->setEnabled(opt.intra);
    ui->spinMinBond->setValue(opt.nminbond);
    ui->checkBoxInter->setCheckState(opt.inter ? Qt::Checked : Qt::Unchecked);
}

void HBondsOptionsDialog::SaveHBoptions()
{
    get_values_for_HBOpt(donorSave,acceptorSave,HBoptSave);
}

void HBondsOptionsDialog::RestoreHBoptions()
{
    set_values_for_HBOpt(donorSave, acceptorSave, HBoptSave);
    int wm = Properties::GetActiveModel();
    puthboptions_(&wm, donorSave, acceptorSave, &HBoptSave);
}
