#include "maps.h"
#include "properties.h"
#if defined(JAV_WIDGET)
#include "crystalviewwidget.h"
#else
#include "javwindow.h"
#endif


extern "C" void PassaMappaParz(char *name, int nmaps, int which, int np[],
          int nrefl, float resm, float denmin, float denmax,
          float avden, float stden, int ifd)
{
   qInfo() << "PASSAMAPPAPARZ:" << QString(name) << nmaps << which << np[0] << nrefl
           << resm << ifd << denmin << denmax << avden << stden;
#if defined(JAV_WIDGET)
    CrystalViewWidget *jav = getJav();
#else
    JavWindow *jav = getJav();
#endif
    jav->PassaMappa(name, nmaps, which, np, nrefl, resm, ifd, denmin, denmax,
              avden, stden);

}

void SetCentreRad(float rad, float *xc=NULL)
{
   if(rad > 0)
      Properties::raggioMappa = rad;
   if(xc != NULL)
   {
       for(int j=0; j<3; j++)
           Properties::centroMappa[j] = xc[j];
       Properties::defCentro = true;
   }
}

extern "C" void map_set_centre(float bar[3]) {
    qInfo() << "BAR: " << bar[0] << bar[1] << bar[2];
    SetCentreRad(-1, bar);
}

extern "C" void map_set_rad(float rad) {
    //qInfo() << "Rad: " << rad;
    SetCentreRad(rad);
}

extern "C" void map_set_visible(int which, int code) {
    //qInfo() << "Which: " << which << " Code: " << code;
#if defined(JAV_WIDGET)
    CrystalViewWidget *jav = getJav();
#else
    JavWindow *jav = getJav();
#endif
    jav->map_set_visible(which, code);
}

extern "C" int is_map_visible(int which) {
    //qInfo() << "Which: " << which;
#if defined(JAV_WIDGET)
    CrystalViewWidget *jav = getJav();
#else
    JavWindow *jav = getJav();
#endif
    int result = (jav->IsMapVisible(which-1))?1:0;
    return result;
}

Mappa::Mappa()
{
      iso = 1000.;
      size_x = 0;
      size_y = 0;
      size_z = 0;
      ifDifference = 1;
      isSurface = false;
      multicolor = false;
      mappa = nullptr;
      mc = nullptr;
      created = false;
      color = nullptr;
      radius = 1.2f;
      step = 0.3f;
      wireframe = 0;
      cx = cy = cz = 0;
      xmin[0] = xmin[1] = xmin[2] = 0;
      xmax[0] = xmax[1] = xmax[2] = 1;
      color_mappa = Properties::MapColors[0];
}

void Mappa::freeMap()
{
    delete mc;
     if(mappa)
        delete [] mappa;
     if(color)
        delete [] color;
     mappa = nullptr;
     color = nullptr;
     created = false;
	
}

void Mappa::SetColor(int ind, bool if_surface)
{
      color_mappa = Properties::MapColors[ind];
      ifVis = !if_surface;
}

void Mappa::FromRadTorad()
{
     radius_x = (int) (Properties::Radius *(float) size_x / Properties::Cell[0] + 0.5f);
     radius_y = (int) (Properties::Radius *(float) size_y / Properties::Cell[1] + 0.5f);
     radius_z = (int) (Properties::Radius *(float) size_z / Properties::Cell[2] + 0.5f);
}

double Mappa::SigmaMappa()
{
      float denmin =  1.0e25f;
      float denmax = -1.0e25f;
      float avden  =  0.0f;
      float stden  =  0.0f;
      for(int i=0;i<size_x*size_y*size_z;i++)
      {
          float xx = mappa[i];
          denmax = max(denmax, xx);
          denmin = min(denmin, xx);
          avden = avden + xx;
          stden = stden + xx * xx;
      }
      float prodotto = (float)(size_x * size_y * size_z);
      avden = avden / prodotto;
      stden = stden / prodotto - avden * avden;
      if (stden > 0.0)
      {
          stden = sqrt( stden);
          sigmaMax = (float) (denmax /stden);
      }
      else
      {
          stden = 0.0;
          sigmaMax = 100.f;
      }
      minimum = (double) denmin;
      maximum = (double) denmax;
      mean = (double) avden;
      return (double) stden;
}

void Mappa::FillMap(int np[], float ***map, int ***ccolor,
                float denmin,float denmax, float avden,float stden)
{
      int size_xx = np[0];
      int size_yy = np[1];
      int size_zz = np[2];
      int size_mm = min(size_xx, size_yy);
      size_mm = min(size_mm, size_zz);
      size_mm /= 2;
      int size_MM = max(size_xx, size_yy);
      size_MM = max(size_MM, size_zz);
      size_MM /= 2;
      Properties::CalcolaMatrici();
      Properties::calcolava();

      size_m = size_mm;
      size_M = size_MM;
      size_x = size_xx;
      size_y = size_yy;
      size_z = size_zz;
      mappa = new float [size_x*size_y*size_z]();
      color = new int [size_x*size_y*size_z]();

      mc = new MarchingCubes();
      mc->set_resolution( size_x, size_y, size_z );
      mc->init_all() ;

      int index_color = 0;
      if(ifDifference < 2)
      {
         for(int kk=0; kk<MAX_NUM_MAPS; kk++)
         {
             if(Properties::usedC[kk] == -1)
             {
                index_color = kk;
                Properties::usedC[kk] = kk;
                break;
             }
         }
      }
      else if(ifDifference == 2)
         index_color = MAX_NUM_MAPS;
      else if(ifDifference == 3)
         index_color = MAX_NUM_MAPS + 1;
      color_mappa = Properties::MapColors[index_color];

      float defRad = Properties::raggioMappa;
      Properties::Radius = defRad;
      if(Properties::defCentro)
         MRVectorCopy(MRVectorMake(Properties::centroMappa), &(Properties::xyzC));
      FromRadTorad();
      int l = 0;
      for(int z=0; z<size_z; z++)
      {
            for(int y=0; y<size_y; y++)
            {
                for(int x=0; x<size_x; x++)
                {
                    if(ccolor != nullptr)
                        color[l] = ccolor[x][y][z];
                    else
                        color[l] = -1;
                    if(map != nullptr)
                        mappa[l++] = (float) map[x][y][z];
                    else if(ifDifference != 3)
                        mappa[l++] = (float) getxmap_(&x, &y, &z);
                    else
                        mappa[l++] = -(float) getxmap_(&x, &y, &z);
                }
            }
      }
      float rap = 1.5f;
      if(fabs(avden) < 1.e-8)
         sigma = (float)SigmaMappa();
      else
      {
         sigma = stden;
         mean  = avden;
         if(stden)
            sigmaMax = (denmax) / (stden);
         else
            sigmaMax = 0;
         minimum = denmin;
         maximum = denmax;
      }
      iso   = rap * sigma;
      if(map != nullptr)
         iso   = 1;
}

void Mappa::CalcolaC()
{
   if((Properties::ifWhole) || isSurface)
      cx = cy = cz = 0;
   else
   {
      double a = (Properties::xyzC.x > 0)?(Properties::xyzC.x):(1+Properties::xyzC.x);
      double b = (Properties::xyzC.y > 0)?(Properties::xyzC.y):(1+Properties::xyzC.y);
      double c = (Properties::xyzC.z > 0)?(Properties::xyzC.z):(1+Properties::xyzC.z);
      if (a > 1)
          a -= 1;
      if (b > 1)
          b -= 1;
      if (c > 1)
          c -= 1;
      cx = (int) ((float) size_x * a +0.5);
      cy = (int) ((float) size_y * b +0.5);
      cz = (int) ((float) size_z * c +0.5);
   }
}

void Mappa::run_mc(int limx, int limy, int limz)
{
  int i, j, k;
  int ii, jj, kk;
  int f;
  int originalMC = 0 ;

  if((Properties::ifWhole) || isSurface)
     f = 1;
  else
     f = 2;

  mc->clean_all() ;
  mc->set_resolution( f*limx+1, f*limy+1, f*limz+1 ) ;
  mc->init_all() ;
  if((limx*limy*limz) == 0)
     return;

  for( k = 0 ; k <= f*limz; k++ )
  {
    kk = (abs(size_z + k - limz + cz)) % size_z;
    kk *= size_x*size_y;

    for( j = 0 ; j <= f*limy ; j++ )
    {
      jj = (abs(size_y + j - limy + cy)) % size_y;
      jj *= size_x;

      for( i = 0 ; i <= f*limx ; i++ )
      {
           ii = (abs(size_x + i - limx + cx)) % size_x;
           int l =  ii + jj + kk;
           if(l >= (size_x*size_y*size_z))
              qDebug() << "Warning";
           double w = (double) mappa[l];
           mc->set_data( w, i,j,k ) ;
           int col = color[l];
           mc->set_color( col, i,j,k ) ;
      }
    }
  }
  double diso = (double) iso;
  mc->set_method( originalMC == 1 ) ;
  mc->run(diso) ;
  mc->clean_temps() ;

  // Rescale positions

  float Xmin = (float)vf[4].x;
  float Ymin = (float)vf[4].y;
  float Zmin = (float)vf[4].z;
  float Xmax = (float)vf[1].x;
  float Ymax = (float)vf[1].y;
  float Zmax = (float)vf[1].z;
  if((Properties::ifWhole) || isSurface)
  {
      Xmin = xmin[0];
      Ymin = xmin[1];
      Zmin = xmin[2];
      Xmax = xmax[0];
      Ymax = xmax[1];
      Zmax = xmax[2];
  }
  float rx = (Xmax - Xmin) /  (f*limx);// - 1);
  float ry = (Ymax - Ymin) /  (f*limy);// - 1);
  float rz = (Zmax - Zmin) /  (f*limz);// - 1);

  for( i = 0 ; i < mc->nverts() ; ++i )
  {
    Vertex &v = mc->vertices()[i] ;
    v.x = rx * v.x + Xmin;
    v.y = ry * v.y + Ymin;
    v.z = rz * v.z + Zmin;
    float nrm =  (float) (v.nx * v.nx + v.ny * v.ny + v.nz * v.nz);
    if(nrm != 0)
    {
       nrm = 1.0f / sqrt(nrm) ;
       v.nx *= nrm ;
       v.ny *= nrm ;
       v.nz *= nrm ;
    }
    //  sembra inutile (ma non lo e')
    //  gestione celle non ortogonali
        vet3 v1 = {{v.x, v.y, v.z}};

    // da coord frazionarie a orto
        vet3 v2 = Properties::FractToCart(v1);
    // coord. grafiche
        v1 = Properties::Trasforma(v2);
        v.x = v1.x;
        v.y = v1.y;
        v.z = v1.z;
  }
}

void Mappa::RunMC()
{
   if(!ifVis)
      return;
   if(!mc)
      return;
   if((Properties::ifWhole) || isSurface)
   {
        if(!isSurface)
           Properties::NormalizzasuCella();
        else
           Properties::NormalizzasuMappa(this);
        run_mc(size_x, size_y, size_z);
   }
   else
   {
        Properties::NormalizzasuMappa(this);
        run_mc(radius_x, radius_y, radius_z);
   }
}

void Mappa::DrawMap(QOpenGLFunctions *f)
{
    float color1[4];
    FromQColor(color_mappa, color1, 4);
    GLenum mode = GL_TRIANGLES;
    int modo[2];
    glGetIntegerv(GL_POLYGON_MODE, modo);
    switch(wireframe)
    {
    case 3:
       mode = GL_POINTS;
    default:
    case 0:
  // setting per wireframe
       glPolygonOffset( 0.5, -0.1f );
       glLineWidth(1.5) ;
       glPolygonMode( GL_FRONT_AND_BACK, GL_LINE ) ;
       if(Properties::Stereo)
          Desaturate(color1);
       glColor4fv(color1);
       f->glDisable (GL_LIGHTING);
       f->glDisable (GL_LINE_SMOOTH);          // piu' veloce
       f->glEnable(GL_BLEND);
       glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
       break;
    case 1:
       SetLightSystem(f, color1, 1, 0);
       break;
    case 2:
       SetLightSystem(f, color1, 0.6f, 0);
       glDepthMask (GL_FALSE);
       break;
    }
    GLdouble *vertices = new GLdouble [9*mc->ntrigs()];
    GLdouble *normals  = new GLdouble [9*mc->ntrigs()];
    fromTrianglesToVertices(vertices, normals);
    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_NORMAL_ARRAY);
    glVertexPointer(3, GL_DOUBLE, 0, vertices);
    glNormalPointer(GL_DOUBLE, 0, normals);
    glDrawArrays(mode, 0, 3*mc->ntrigs());
    glDisableClientState(GL_VERTEX_ARRAY);
    glDisableClientState(GL_NORMAL_ARRAY);
    delete [] vertices;
    delete [] normals;
    glLineWidth(1.0) ;
    glDepthMask (GL_TRUE);
    glPolygonMode( GL_FRONT_AND_BACK, modo[0] ) ;
    f->glDisable(GL_BLEND);
    f->glEnable (GL_LIGHTING);
}

void Mappa::fromTrianglesToVertices(GLdouble *vertices, GLdouble *normals)
{
    int k = 0;
    for( int i = 0 ; i < mc->ntrigs() ; ++i )
    {
      const Triangle *t = mc->trig(i) ;
      const Vertex   *v ;
      v = mc->vert(t->v1) ;
//      if(v == nullptr)
//          continue;
      vertices[k]   = v->x;
      vertices[k+1] = v->y;
      vertices[k+2] = v->z;
      normals[k]    = v->nx;
      normals[k+1]  = v->ny;
      normals[k+2]  = v->nz;
      k += 3;
      v = mc->vert(t->v2) ;
//      if(v == nullptr)
//          continue;
      vertices[k]   = v->x;
      vertices[k+1] = v->y;
      vertices[k+2] = v->z;
      normals[k]    = v->nx;
      normals[k+1]  = v->ny;
      normals[k+2]  = v->nz;
      k += 3;
      v = mc->vert(t->v3) ;
//      if(v == nullptr)
//          continue;
      vertices[k]   = v->x;
      vertices[k+1] = v->y;
      vertices[k+2] = v->z;
      normals[k]    = v->nx;
      normals[k+1]  = v->ny;
      normals[k+2]  = v->nz;
      k += 3;
      //qDebug( ) << "k " << k << ", trigs " << mc->ntrigs() << &endl;
    }
}
