#include "normhydrodlg.h"
#include "ui_normhydrodlg.h"
#include "floatdelegate.h"

NormHydroDlg::NormHydroDlg(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::NormHydroDlg)
{
    QStringList titles = {"Atom Type", "Bond Length"};
    ui->setupUi(this);
    mModel = new QStandardItemModel(this);
    mModel->setHorizontalHeaderLabels(titles);
    FloatDelegate *delegate = new FloatDelegate(ui->HydroView);
    delegate->setDecimals(2);
    ui->HydroView->setItemDelegateForColumn(1, delegate);
}

NormHydroDlg::~NormHydroDlg()
{
    delete ui;
}

void NormHydroDlg::showEvent(QShowEvent *event)
{
    populateModel();
    QDialog::showEvent(event);
}

void NormHydroDlg::populateModel()
{
    QStringList firstC = {"Methyl group CH3",
                          "Methilene group CH3",
                          "Methine CH",
                          "Acetylenic CH",
                          "Aromatic CH",
                          "Terminal X=CH2",
                          "O-H",
                          "N-H in primary amine",
                          "N-H in secondary amine",
                          "S-H",
                          "B-H"};

    int nRows = mModel->rowCount();
    mModel->removeRows(0, nRows);
    int nlines = firstC.count();
    for(int i=0; i<nlines; i++)
    {
        QList <QStandardItem *> items;
        QStandardItem *item = new QStandardItem(QString(firstC.at(i)));
        item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
        items.append(item);
        item = new QStandardItem();
        item->setData(Hdist[i], Qt::DisplayRole);
        items.append(item);
        mModel->appendRow(items);
    }
    ui->HydroView->setModel(mModel);
    ui->HydroView->horizontalHeader()->setStretchLastSection(false);
    ui->HydroView->resizeColumnsToContents();
    ui->HydroView->horizontalHeader()->setStretchLastSection(true);
}

float *NormHydroDlg::getDistances()
{
    float *v = new float [nDistances]();
    int nrows = mModel->rowCount();
    for(int i=0; i<nrows; i++)
    {
        Hdist[i] = mModel->item(i, 1)->data(Qt::DisplayRole).toFloat();
        v[i] = Hdist[i];
    }
    return v;
}

void NormHydroDlg::on_okButton_clicked()
{
    int code = 1;
    emit normHydroDlgClosed(code, nDistances);
    accept();
}

void NormHydroDlg::on_applyButton_clicked()
{
    int code = 2;
    emit normHydroDlgClosed(code, nDistances);
}

void NormHydroDlg::on_defaultsButton_clicked()
{
    for(int i=0; i<nDistances; i++)
        Hdist[i] = distH0[i];
    populateModel();
}

void NormHydroDlg::on_cancelButton_clicked()
{
    int code = 3;
    emit normHydroDlgClosed(code, nDistances);
    reject();
}
