#include "pairdist.h"
#include "ui_pairdist.h"
#include "savedialog.h"
#include "qt_utils.h"
#include "javkeysettings.h"
#include <QResizeEvent>
#include <QStatusBar>
#include <QMessageBox>
#include <QSettings>
#include <QToolTip>

PairDist *PDFDialog;

extern "C" void jav_export_powder_pattern_(char *f1, int *tipo, size_t lung);

QString PairDist::MessForSB(QPointF p)
{
    return PDFDialog->StatusMessage(p);
}

PairDist::PairDist(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::PairDist),
    m_pdf({0, 10.f, 20.f}),
    m_Scene(nullptr),
    mPDFsett(nullptr)
{
    ui->setupUi(this);
    mStatusBar = new QStatusBar(this);
    mStatusBar->setSizeGripEnabled(false);
    ui->verticalLayout->insertWidget(1,mStatusBar);
    ui->resetButton->setEnabled(false);
    PDFDialog = this;
    if(m_Scene != nullptr)
       delete m_Scene;
    m_Scene = new Grafico(mStatusBar, ui->resetButton, this);
    m_Scene->SetFunc(PairDist::MessForSB);
}

PairDist::~PairDist()
{
    delete ui;
}

void PairDist::resizeEvent(QResizeEvent *)
{
    QRectF rect1(ui->graphicsView->rect());
    m_Scene->sizeChanged(rect1.width(), rect1.height());
}

QString PairDist::StatusMessage(QPointF p)
{
    QString stringa;
    float r, g;

    r = p.x();
    g = p.y();
    stringa = QString("R: %1      G: %2").arg(r).arg(g);

    return  stringa;
}

void PairDist::on_settingsButton_clicked()
{
    if(!mPDFsett)
       mPDFsett = new PairDistSettings(mStatusBar, this);
    mPDFsett->FillForm(m_Kmodel, &m_pdf);
    mPDFsett->show();
}

void PairDist::on_resetButton_clicked()
{
    m_Scene->ResetZoom();
    ui->resetButton->setEnabled(false);
}

void PairDist::on_closeButton_clicked()
{
   reject();
}

void PairDist::on_exportButton_clicked()
{
    QSettings settings;
    QString selectedFilter = "ASCII powder pattern(*.xy)";
    QString fileName = SaveDialog::run(this, tr("Export pattern as"),
                                    settings.value(DEFAULT_DIR_KEY).toString(),
                                       QFileInfo(m_FileName).baseName(),
                                       "ASCII powder pattern(*.xy)",
                                       "xy", selectedFilter);
    if (!fileName.isEmpty()) {
         QFile file(fileName);
         if (!file.open(QFile::WriteOnly | QFile::Text)) {
             QMessageBox::warning(this, "Warning", "Cannot save file: " + file.errorString());
             return;
         }
         file.close();
         int tipo = 1;
         char *f1 = ToCharacter(QFileInfo(fileName).absoluteFilePath());
         jav_export_powder_pattern_(f1, &tipo, strlen(f1));
    }
}

void PairDist::FillPDFDlg(int whichModel, QString filename)
{
    m_Kmodel = whichModel;
    m_FileName = filename;
    m_Scene->ClearScene();
    Attesa("Computing PDF. Please wait ...", mStatusBar);
    jav_pdf_(&m_Kmodel, &m_pdf);
}

void PairDist::DoPDF(float x[], float y[], int num)
{
    FineAttesa(mStatusBar);
    //QString Aring = QString(0x0c5).toUtf8();
    m_Scene->ClearScene();
    m_Scene->SetArea(ui->graphicsView->width()-10, ui->graphicsView->height()-10);
    m_Scene->setSceneRect(ui->graphicsView->rect());
    QString labelx("R(");
    labelx.append(Aring());
    labelx.append(")");
    QString labely("G(");
    labely.append(Aring());
    labely.append("<sup>-2</sup>)");
    m_Scene->SetLabels(labelx, labely);
    m_Scene->AddPlot(x, y, num, 0, 0, false);
    m_Scene->SetArea(ui->graphicsView->width(), ui->graphicsView->height());
    m_Scene->Aggiorna();
    ui->graphicsView->setScene(m_Scene);
}

extern "C" void do_pdf_(float x[], float y[], int *num)
{
    PDFDialog->DoPDF(x, y, *num);
}
