#include "phasewidget.h"
#include "ui_phasewidget.h"

#include <QDebug>

extern "C" void get_crystal_data(int idph, crydata::crystal_t *ccryst);
extern "C" void set_crystal_data(int idph, crydata::crystal_t *ccryst);
extern "C" int crystal_atom_number(int id);
extern "C" void get_crystal_atoms(int id, crydata::c_atom_type atm[]);
extern "C" void put_crystal_atoms(int id, crydata::c_atom_type atm[], int coord);
extern "C" void get_info_po(int idph, int hkl[3], float *gpar, int *code);
extern "C" void put_info_po(int idph, int hkl[3], float gpar, int code);
extern "C" void get_symmetry_info(int phaseid, int *spgIndex, char *spaceg, double cell[6]);
extern "C" void put_cell_info(int phaseid, double c_cell[], int coderef);


//extern "C" int scatters_number(int idph);
//extern "C" void get_scatter_info(int idph, int kscatt, char *lab);

PhaseWidget::PhaseWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::PhaseWidget),
    phaseId(-1)
{
    ui->setupUi(this);
}

PhaseWidget::~PhaseWidget()
{
    delete ui;
}

void PhaseWidget::setWidget(int id)
{
    phaseId = id;

    get_crystal_data(phaseId, &cryst);
    ui->profileTab->setData(phaseId, cryst);

    int hklPO[3];
    int codePO;
    float parPO;
    get_info_po(phaseId, hklPO, &parPO, &codePO);
    ui->correctionTab->setData(hklPO, parPO, codePO);

    int spgIndex;
    char *spaceg = new char[40];
    double cell[6];
    get_symmetry_info(phaseId, &spgIndex, spaceg, cell);
    ui->structureTab->setSymmetry(phaseId, cell, QString::fromUtf8(spaceg));

    int nat = crystal_atom_number(phaseId);
    auto atm = new crydata::c_atom_type[nat];
    get_crystal_atoms(phaseId, atm);
    QVector<crydata::AtomProperties> atoms(nat);
    for (int i = 0; i < nat; i++) {
        atoms[i].label = QString::fromUtf8(atm[i].lab);
        for (int j = 0; j < 3; j++) atoms[i].xyz[j] = atm[i].xyz[j];
        atoms[i].biso = atm[i].biso;
        atoms[i].occ = atm[i].occ;
        for (int j = 0; j < 5; j++) atoms[i].rcod[j] = atm[i].rcod[j];
        atoms[i].kscatt = atm[i].kscatt - 1;
    }

    QVector<crydata::ConstraintProperties> constr;
    crydata::make_constraints(phaseId, constr);
    ui->structureTab->setConstraints(constr);
    ui->structureTab->setAtoms(atoms);
}

void PhaseWidget::getData()
{
    ui->profileTab->getData(phaseId, cryst);
    set_crystal_data(phaseId, &cryst);

    int hklPO[3];
    int codePO;
    float parPO;
    ui->correctionTab->getData(hklPO, parPO, codePO);
    put_info_po(phaseId, hklPO, parPO, codePO);

    double cell[6];
    bool cellref;
    ui->structureTab->getSymmetry(cell,cellref);
    put_cell_info(phaseId, cell, cellref);

//    int nat = crystal_atom_number(phaseId);
//    QVector<AtomProperties> atoms(nat);
//    ui->structureTab->getAtoms(atoms);
//    auto atm = new crydata::c_atom_type[nat];
//    for (int i = 0; i < nat; i++) {
//         for (int j = 0; j < 3; j++) atm[i].xyz[j] = atoms[i].xyz[j];
//         atm[i].biso = atoms[i].biso;
//         atm[i].occ = atoms[i].occ;
//         for (int j = 0; j < 5; j++) atm[i].rcod[j] = atoms[i].rcod[j];
//    }
//    put_crystal_atoms(phaseId, atm, 0);
}

bool PhaseWidget::hasParameterToRefine()
{
    if (ui->profileTab->hasParameterToRefine()) return true;
    if (ui->correctionTab->hasParameterToRefine()) return true;
    if (ui->structureTab->hasParameterToRefine()) return true;

    return false;
}

bool PhaseWidget::hasProfileFunctionChanged()
{
    return ui->profileTab->hasParameterChanged();
}

bool PhaseWidget::hasPOChanged()
{
    return ui->correctionTab->hasPOChanged();
}

bool PhaseWidget::hasCellChanged()
{
    return ui->structureTab->hasCellChanged();
}

void PhaseWidget::setRefineEnabled(bool enable)
{
    ui->profileTab->setRefineEnabled(enable);
    ui->structureTab->setRefineEnabled(enable);
}
