#include "processdialog.h"
#include "ui_processdialog.h"

#include <QMessageBox>
#include <QDebug>

ProcessDialog::ProcessDialog(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::ProcessDialog),
    programTerminated(false)
{
    ui->setupUi(this);
    mProcess = new QProcess(this);
    mProcess->setProcessChannelMode(QProcess::MergedChannels);

    connect(mProcess, &QProcess::readyReadStandardOutput, this, &ProcessDialog::displayOutput);    
    connect(mProcess, &QProcess::finished, this, &ProcessDialog::processFinished);
    connect(mProcess, &QProcess::errorOccurred, this, &ProcessDialog::processError);
    connect(ui->buttonBox, &QDialogButtonBox::clicked, this, &ProcessDialog::onButtonBoxClicked);
}

ProcessDialog::~ProcessDialog()
{
    delete ui;
}

void ProcessDialog::setProgram(const QString &program)
{
    mProcess->setProgram(program);
    if (programName.isEmpty()) setProgramName(program);
}

void ProcessDialog::setArguments(const QStringList &arguments)
{
    mProcess->setArguments(arguments);
}

void ProcessDialog::start()
{
    mProcess->start();
}

void ProcessDialog::waitAndProcessEvents()
{
    while (1) {
        QCoreApplication::processEvents();
        if ( mProcess->state() == QProcess::NotRunning) break;
    }
}

QProcess::ProcessState ProcessDialog::state() const
{
    return mProcess->state();
}

QProcess *ProcessDialog::process() const
{
    return mProcess;
}

void ProcessDialog::displayOutput()
{
    ui->textEdit->append(mProcess->readAllStandardOutput());
}

void ProcessDialog::processFinished()
{
    emit processDialogFinished();
}

void ProcessDialog::onButtonBoxClicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if(stdButton == QDialogButtonBox::Abort) {
        terminateProcess();
    }
}

void ProcessDialog::processError(QProcess::ProcessError error)
{
    Q_UNUSED(error);
    if (!programTerminated) // avoid message when program is terminated by user
        QMessageBox::warning(this, QObject::tr("%1 error").arg(programName),mProcess->errorString());
}

void ProcessDialog::setProgramName(const QString &newProgramName)
{
    programName = newProgramName;
    this->setWindowTitle(programName);
}

void ProcessDialog::terminateProcess()
{
    // parentwidget is used because if 'this' is non shown the meggasebox is uncorrectly positioned
    int ret = QMessageBox::question(this->parentWidget(), programName, "Would Would you like to terminate "+programName+ "?");
    if (ret == QMessageBox::Yes) {
        programTerminated = true;
        //mProcess->terminate();
        mProcess->kill();
    }
}
