#include "spheredialog.h"
#include "ui_spheredialog.h"

SphereDialog::SphereDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::SphereDialog),
    kmodel(1)
{
    ui->setupUi(this);
    QStringList headers = { "Number", "Atom", "Distance", "Symmetry Operator"};
    sphereList = CreaLista(headers);
    ui->tableView->setModel(sphereList);
    ui->tableView->setColumnHidden(0, true);
    ui->tableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    ui->tableView->setSelectionMode(QAbstractItemView::MultiSelection);
    ui->tableView->verticalHeader()->hide();
    ui->tableView->setAlternatingRowColors(true);
    ui->tableView->setShowGrid(false);
    ui->tableView->resizeColumnsToContents();
}

SphereDialog::~SphereDialog()
{
    delete ui;
}

void SphereDialog::reject()
{
    int nrows = sphereList->rowCount();
    sphereList->removeRows(0, nrows);
    ui->centre->blockSignals(true);
    ui->centre->setText("");
    ui->centre->blockSignals(false);
    QDialog::reject();
}

void SphereDialog::rowSelectionChanged(const QItemSelection &, const QItemSelection &)
{
    QVector<int> selectedRows;
    QModelIndexList selection = ui->tableView->selectionModel()->selectedIndexes();
    for (const QModelIndex &index : selection) {
       if(index.column() == 0)  {
           int row = sphereList->data(index, Qt::DisplayRole).toInt();
           selectedRows.push_back(row);
       }
    }
    emit(sphereSelectionChanged(selectedRows));
}

void SphereDialog::SetCentre(QString lab_atom)
{
    ui->centre->blockSignals(true);
    ui->centre->setText(lab_atom);
    ui->centre->blockSignals(false);
}

void SphereDialog::SetRadius(float val)
{
    ui->spinRadius->blockSignals(true);
    ui->spinRadius->setValue(val);
    ui->spinRadius->blockSignals(false);
}

void SphereDialog::on_spinRadius_valueChanged(double val)
{
    emit(selectedRadius((float)val));
}

void SphereDialog::on_centre_editingFinished()
{
    QString lab = ui->centre->text();
    emit(selectedCentre (lab));
}

QStandardItemModel *SphereDialog::CreaLista(QStringList headers)
{
    QStandardItemModel *model = new QStandardItemModel(this);
    model->setHorizontalHeaderLabels(headers);
    return model;
}

void SphereDialog::fillList(QList<QStandardItem *> *items, int nitems)
{
    int nrows = sphereList->rowCount();
    sphereList->removeRows(0, nrows);
    for(int i=0; i<nitems; i++)
       sphereList->appendRow(items[i]);
    ui->tableView->resizeColumnsToContents();
    QItemSelectionModel *selectionModel = ui->tableView->selectionModel();
    connect(selectionModel, &QItemSelectionModel::selectionChanged,
           this, &SphereDialog::rowSelectionChanged);
}
