#include <QOpenGLWidget>
#include <QtGui/QOpenGLFunctions>
#include <QFileInfo>
#include "stlutils.h"

STL_class stl_file;

size_t read_project(ifstream &f, char *str, streamsize dime)
{
    size_t nsz = 0;
    streamsize sz = f.rdbuf()->sgetn(str, dime);
    if(sz >= 0)
        nsz = static_cast<size_t>(sz);
    return nsz;
}

void get_double_tokens (string str, char c, size_t num_tokens, double my_tokens[])
{

    vector <string> tokens; 
    stringstream check1(str); 
    string intermediate; 
      
    while(getline(check1, intermediate, c)) 
    { 
        tokens.push_back(intermediate); 
    } 
    size_t ifinal = min(num_tokens, tokens.size());
    for(size_t i=0; i<ifinal; i++)
        my_tokens[i] = atof(tokens.at(i).c_str());
}

bool IsSTLBinary(string &filename)
{
    bool binary=false;
    ifstream is;
    int facenum;
    is.open(filename.c_str(), ios::in);
    is.seekg (0, is.end);
    int file_size = static_cast<int>(is.tellg());
    is.seekg (0, is.beg);
    is.seekg(HEADER_SIZE);
    size_t sz = read_project(is, static_cast<char*>(static_cast<void*>(&facenum)), sizeof(int));
    if(sz != sizeof(int))
        binary = false;
    else
    {
        //int facet_size = sizeof(Triangolo) - sizeof(int) + sizeof(short);
        int facet_size = 12*sizeof(float) + sizeof(short);
        int expected_file_size = HEADER_SIZE + 4 + facet_size * facenum;
        if(file_size ==  expected_file_size)
           binary = true;
    }
    is.close();
    return binary;
}

static void normalize(GLdouble *a)
{
    GLdouble d=sqrt(a[0]*a[0]+a[1]*a[1]+a[2]*a[2]);
    a[0]/=d; a[1]/=d; a[2]/=d;
}

void STL_class::PrintSferaSTL(int nat, double radius, int ndiv)
{
    static GLdouble vdata[12][3] = {
       {-XC, 0.0, ZC}, {XC, 0.0, ZC}, {-XC, 0.0, -ZC}, {XC, 0.0, -ZC},
       {0.0, ZC, XC}, {0.0, ZC, -XC}, {0.0, -ZC, XC}, {0.0, -ZC, -XC},
       {ZC, XC, 0.0}, {-ZC, XC, 0.0}, {ZC, -XC, 0.0}, {-ZC, -XC, 0.0}
   };
   static GLuint tindices[20][3] = {
       {0,4,1}, {0,9,4}, {9,5,4}, {4,5,8}, {4,8,1},
       {8,10,1}, {8,3,10}, {5,3,8}, {5,2,3}, {2,7,3},
       {7,10,3}, {7,6,10}, {7,11,6}, {11,0,6}, {0,1,6},
       {6,1,10}, {9,0,11}, {9,11,2}, {9,2,5}, {7,2,11} };
   ndiv = ndiv/50 + 2;
   if(ndiv > 4)
      ndiv = 4;
   for (int i=0;i<20;i++)
        drawtri(vdata[tindices[i][0]], vdata[tindices[i][1]], vdata[tindices[i][2]], ndiv, radius, nat);
}

void STL_class::drawtri(GLdouble *a, GLdouble *b, GLdouble *c, int div, double r, int nat)
{
    if (div<=0) {
        
        Triangolo t;
        for(int j=0; j<3; j++)
        {
           t.v1[j] = a[j]*r;
           t.v2[j] = b[j]*r;
           t.v3[j] = c[j]*r;
        }
        t.nat = nat;
        trigs.push_back(t);
    } else {
        GLdouble ab[3], ac[3], bc[3];
        for (int i=0;i<3;i++) {
            ab[i]=(a[i]+b[i])/2;
            ac[i]=(a[i]+c[i])/2;
            bc[i]=(b[i]+c[i])/2;
        }
        normalize(ab); normalize(ac); normalize(bc);
        drawtri(a, ab, ac, div-1, r, nat);
        drawtri(b, bc, ab, div-1, r, nat);
        drawtri(c, ac, bc, div-1, r, nat);
        drawtri(ab, bc, ac, div-1, r, nat);
    }  
}

bool STL_class::readSTL_ASCII(vector<double> &tvet)
{
  ifstream f;
  bool cicla = true;
  char line[256];
  Triangolo *trig = new Triangolo;
  int nv = 0;

  f.open(filename.c_str(), ios::in);
  if(!f.is_open() || f.fail())
     goto call_error;
  while(cicla)
  {
     double v1[3];
     f.getline(line, 256);
     if ( (f.rdstate() & ifstream::eofbit ) != 0 )
         cicla = false;
     if(strstr(line, "solid ") || strstr(line, "endsolid"))
        continue;
     if(strstr(line, "facet normal"))
     {
        string linestr = line;
        size_t p = linestr.find("facet normal");
        linestr.erase(0, p+12);
        get_double_tokens(linestr,  ' ', 3,  v1);
        for (int j=0; j<3; j++)
             trig->vn[j] = v1[j];
        trig->nat = -1;
     }
     if(strstr(line, "vertex"))
     {
        string linestr = line;
        size_t p = linestr.find("vertex");
        linestr.erase(0, p+6);
        get_double_tokens(linestr,  ' ', 3,  v1);
        for (int j=0; j<3; j++)
        {
             if(nv == 1)
                trig->v1[j] = v1[j];
             else if(nv == 2)
                trig->v2[j] = v1[j];
             else if(nv == 3)
                trig->v3[j] = v1[j];
             tvet.push_back(v1[j]);
        }
        nv++;
        trig->nat = -1;
     }
     if(strstr(line, "outer loop"))
        nv = 1;
     if(strstr(line, "endloop"))
        nv = 0;
     if(strstr(line, "endfacet"))
        trigs.push_back(*trig);
  }
   
  f.close();
  return true;
call_error:
  if(f.is_open())
     f.close();
  return false;
}

bool STL_class::readSTL_BIN(vector<double> &tvet)
{
  char header[81];
  unsigned short int at =  0;
  unsigned int ntrigs;
  ifstream f;
  size_t nsz;

  f.open(filename.c_str(), ios::binary | ios::in);
  if(!f.is_open() || f.fail())
     goto call_error;
  nsz = read_project(f, header, HEADER_SIZE);
  if((nsz != HEADER_SIZE) || f.fail())
     goto call_error;
  header[HEADER_SIZE] = 0;
  nsz = read_project(f, static_cast<char*>(static_cast<void*>(&ntrigs)), 4);
  if((nsz != 4) || f.fail())
     goto call_error;
  for (size_t i=0; i<ntrigs; i++)
  {
       float v1[3], v2[3], v3[3], vet[3];
       nsz = read_project(f, static_cast<char*>(static_cast<void*>(vet)), 3*sizeof(float));
       if((nsz != (3*sizeof(float)) || f.fail()))
          goto call_error;
       nsz = read_project(f, static_cast<char*>(static_cast<void*>(v1)), 3*sizeof(float));
       if((nsz != (3*sizeof(float)) || f.fail()))
          goto call_error;
       nsz = read_project(f, static_cast<char*>(static_cast<void*>(v2)), 3*sizeof(float));
       if((nsz != (3*sizeof(float)) || f.fail()))
          goto call_error;
       nsz = read_project(f, static_cast<char*>(static_cast<void*>(v3)), 3*sizeof(float));
       if((nsz != (3*sizeof(float)) || f.fail()))
          goto call_error;
       nsz = read_project(f, static_cast<char*>(static_cast<void*>(&at)), sizeof(at));
       if((nsz != (sizeof(short)) || f.fail()))
          goto call_error;
       Triangolo tri;
       for(int k=0; k<3; k++)
       {
           tri.v1[k] = static_cast<double>(v1[k]);
           tri.v2[k] = static_cast<double>(v2[k]);
           tri.v3[k] = static_cast<double>(v3[k]);
           tri.vn[k] = static_cast<double>(vet[k]);
           tvet.push_back(tri.v1[k]);
           tvet.push_back(tri.v2[k]);
           tvet.push_back(tri.v3[k]);
       }
       tri.nat = -1;
       trigs.push_back(tri);
  }
  f.close();
  return true;
call_error:
  if(f.is_open())
     f.close();
  return false;
}

bool STL_class::readSTL()
{
  vector<double> tvet;
  double mini=0, maxi=0, delta=0;
  bool result;

  trigs.clear();
  tvet.clear();
  if(!IsSTLBinary(filename))
     result = readSTL_ASCII(tvet);
  else
     result = readSTL_BIN(tvet);

  if(!result)
     return false;

  if(!tvet.empty())
  {
      mini = (*min_element(tvet.begin(), tvet.end()));
      maxi = (*max_element(tvet.begin(), tvet.end()));
      delta = maxi-mini;
      tvet.clear();
  }

  if(!trigs.empty())
     nTrigs = trigs.size();
  else
     nTrigs = 0;

  if(abs(delta) > 1e-7)
  {
     double fact = 2 / delta;
     for (size_t i=0; i<nTrigs; i++)
     {
          Triangolo *tri = &(trigs.at(i));
          for(int k=0; k<3; k++)
          {
              tri->v1[k] = -1 + (tri->v1[k] - mini)*fact;
              tri->v2[k] = -1 + (tri->v2[k] - mini)*fact;
              tri->v3[k] = -1 + (tri->v3[k] - mini)*fact;
          }
     }
  }
  return true;
  
}

void STL_class::writeSTL(std::vector<vet3> &xyz)
{
  nTrigs = trigs.size();
  std::string f_ascii = filename;
  size_t n = f_ascii.find_last_of('.');
  if(n != std::string::npos)
     f_ascii = f_ascii.erase(n, std::string::npos);
  f_ascii.append("_ascii.stl");
  std::ofstream f_a;
  f_a.open(f_ascii.c_str(), std::ios::out);
  std::ofstream f;
  f.open(filename.c_str(), std::ios::binary | std::ios::out);
  QFileInfo fi(filename.c_str());
  std::string title = fi.baseName().toStdString();
  n = title.find_last_of('.');
  if(n != std::string::npos)
     title = title.erase(n, std::string::npos);

  f_a <<  "solid " << title << "\n";
  const char header[81] =
"made by Jav                                                                     ";
  unsigned short int at =  0;
  f.write(header, HEADER_SIZE);
  f.write(static_cast<const char*>(static_cast<const void*>(&nTrigs)), 4);
  for (size_t i=0; i<nTrigs; i++)
  {
       int nat = trigs.at(i).nat;
       float vc[3] = {2, 2, 2};
       if(nat > -1)
       {
          for(int j=0; j<3; j++)
              vc[j] += (float) xyz[nat].v[j];
       }
       float v1[3] = {(float)trigs.at(i).v1[0]+vc[0], (float)trigs.at(i).v1[1]+vc[1], (float)trigs.at(i).v1[2]+vc[2]};
       float v2[3] = {(float)trigs.at(i).v2[0]+vc[0], (float)trigs.at(i).v2[1]+vc[1], (float)trigs.at(i).v2[2]+vc[2]};
       float v3[3] = {(float)trigs.at(i).v3[0]+vc[0], (float)trigs.at(i).v3[1]+vc[1], (float)trigs.at(i).v3[2]+vc[2]};
       Vert p1 = { v1[0], v1[1], v1[2] , {0, 0, 0, 0}};
       Vert p2 = { v2[0], v2[1], v2[2] , {0, 0, 0, 0}};
       Vert p3 = { v3[0], v3[1], v3[2] , {0, 0, 0, 0}};
       Vert n = norm(p1, p2, p3);
       float vet[3] =  {n.x, n.y, n.z};
       f_a << "facet normal " << n.x << n.y << n.z << "\n";
       f_a << "    outer loop\n";
       f_a << "       vertex " << v1[0] << v1[1] << v1[2] << "\n";
       f_a << "       vertex " << v2[0] << v2[1] << v2[2] << "\n";
       f_a << "       vertex " << v3[0] << v3[1] << v3[2] << "\n";
       f_a << "    endloop\n";
       f_a << "endfacet\n";
       f.write(static_cast<const char*>(static_cast<const void*>(vet)), 3*sizeof(float));
       f.write(static_cast<const char*>(static_cast<const void*>(v1)), 3*sizeof(float));
       f.write(static_cast<const char*>(static_cast<const void*>(v2)), 3*sizeof(float));
       f.write(static_cast<const char*>(static_cast<const void*>(v3)), 3*sizeof(float));
       f.write(static_cast<const char*>(static_cast<const void*>(&at)), sizeof(at));
  }
  f.close();
  f_a << "endsolid sfera\n";
  f_a.close();
}

void STL_class::Pol2Trigs(vet3 v[4])
{
    Triangolo trig;
    for(int j=0; j<3; j++)
    {
           trig.v1[j] = (float) v[0].v[j];
           trig.v2[j] = (float) v[1].v[j];
           trig.v3[j] = (float) v[2].v[j];
    }
    trig.nat = -1;
    trigs.push_back(trig);
    for(int j=0; j<3; j++)
    {
           trig.v1[j] = (float) v[2].v[j];
           trig.v2[j] = (float) v[3].v[j];
           trig.v3[j] = (float) v[0].v[j];
    }
    trig.nat = -1;
    trigs.push_back(trig);
}

void STL_class::TFVert2Tri(std::vector<vet3> Vertici)
{
  for(int i=0; i< (int) Vertici.size()-2; i++)
  {
       Triangolo trig;
       for(int j=0; j<3; j++)
       {
           trig.v1[j] = Vertici.at(0).v[j];
           trig.v2[j] = Vertici.at(i+1).v[j];
           trig.v3[j] = Vertici.at(i+2).v[j];
       }
       trig.nat = -1;
       trigs.push_back(trig);
  }
}

void STL_class::QSVert2Tri(std::vector<vet3> Vertici)
{
    int nq = (int) Vertici.size();
    for(int i=0; i< nq/2-1; i++)
    {
         int n = 2*i;
         int n1 = (n+1) % nq;
         int n2 = (n+2) % nq;
         int n3 = (n+3) % nq;
         Triangolo trig;
         for(int j=0; j<3; j++)
         {
             trig.v1[j] = Vertici.at(n).v[j];
             trig.v2[j] = Vertici.at(n1).v[j];
             trig.v3[j] = Vertici.at(n3).v[j];
         }
         trig.nat = -1;
         trigs.push_back(trig);
         for(int j=0; j<3; j++)
         {
             trig.v1[j] = Vertici.at(n3).v[j];
             trig.v2[j] = Vertici.at(n2).v[j];
             trig.v3[j] = Vertici.at(n).v[j];
         }
         trig.nat = -1;
         trigs.push_back(trig);
    }
}

void STL_class::PrintCilindroSTL(vet3 p1, vet3 p2, double raggio)
{
  vet3 p1p2N = MRVectorSub(p1, p2);
  vet3 firstPerpV = MRVectorFirstPerp(p1p2N);
  vet3 secondPerpV = MRVectorCrossProduct(p1p2N, firstPerpV);
  secondPerpV = MRVectorNormalize(secondPerpV);
  const int ANZ = 32;
  const float FULL = (float) (2.0f*M_PI);
  vet3 points[ANZ+1];

  for (int i=0; i<ANZ; i++)
  {
    float angle = FULL*((float) i/(float) ANZ);

    points[i].x = (float) (raggio*(cos(angle)*firstPerpV.x+sin(angle)*secondPerpV.x));
    points[i].y = (float) (raggio*(cos(angle)*firstPerpV.y+sin(angle)*secondPerpV.y));
    points[i].z = (float) (raggio*(cos(angle)*firstPerpV.z+sin(angle)*secondPerpV.z));
  }
  points[ANZ].x = points[0].x;
  points[ANZ].y = points[0].y;
  points[ANZ].z = points[0].z;
  std::vector<vet3> Vertici;
  Vertici.push_back(p1);
  for (int i=0; i<=ANZ; i++)
  {
       vet3  p = { {p1.x+points[i].x, p1.y+points[i].y, p1.z+points[i].z} };
       Vertici.push_back(p);
  }
  TFVert2Tri(Vertici);
  Vertici.clear();
  Vertici.push_back(p2);
  for (int i=0; i<=ANZ; i++)
  {
       vet3  p = { {p2.x+points[i].x, p2.y+points[i].y, p2.z+points[i].z} };
       Vertici.push_back(p);
  }
  TFVert2Tri(Vertici);
  Vertici.clear();
  for (int i=0; i<=ANZ; i++)
  {
       vet3  p = { {p1.x+points[i].x, p1.y+points[i].y, p1.z+points[i].z} };
       Vertici.push_back(p);
       vet3  pp = { {p2.x+points[i].x, p2.y+points[i].y, p2.z+points[i].z} };
       Vertici.push_back(pp);
  }
  QSVert2Tri(Vertici);
}

void STL_class::DrawTriangles(QOpenGLFunctions *m_context)
{
    float color1[4] = {0.85f, 0.85f, 0.85f, 1};
    m_context->glEnable (GL_LIGHTING);
    m_context->glEnable (GL_NORMALIZE);
    m_context->glEnable (GL_BLEND);
    m_context->glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color1);
    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL ) ;

    GLenum mode = GL_TRIANGLES;
    GLdouble *vertices = new GLdouble[9*nTrigs]();
    GLdouble *normals  = new GLdouble[9*nTrigs]();

    int k = 0;
    for( size_t i = 0 ; i < nTrigs ; ++i )
    {
      Triangolo t = trigs.at(i);
      vet3 v1 = {{t.v1[0], t.v1[1], t.v1[2]}};
      vet3 v2 = {{t.v2[0], t.v2[1], t.v2[2]}};
      vet3 v3 = {{t.v3[0], t.v3[1], t.v3[2]}};
      vet3 vn = {{t.vn[0], t.vn[1], t.vn[2]}};

      vertices[k]   = v1.x;
      vertices[k+1] = v1.y;
      vertices[k+2] = v1.z;
      normals[k]    = vn.x;
      normals[k+1]  = vn.y;
      normals[k+2]  = vn.z;
      k += 3;
      vertices[k]   = v2.x;
      vertices[k+1] = v2.y;
      vertices[k+2] = v2.z;
      normals[k]    = vn.x;
      normals[k+1]  = vn.y;
      normals[k+2]  = vn.z;
      k += 3;
      vertices[k]   = v3.x;
      vertices[k+1] = v3.y;
      vertices[k+2] = v3.z;
      normals[k]    = vn.x;
      normals[k+1]  = vn.y;
      normals[k+2]  = vn.z;
      k += 3;
    }
    m_context->glEnable(GL_CULL_FACE);
    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_NORMAL_ARRAY);
    glVertexPointer(3, GL_DOUBLE, 0, vertices);
    glNormalPointer(GL_DOUBLE, 0, normals);
    m_context->glDrawArrays(mode, 0, 3*static_cast<int>(nTrigs));
    glDisableClientState(GL_VERTEX_ARRAY);
    glDisableClientState(GL_NORMAL_ARRAY);
    m_context->glDisable(GL_CULL_FACE);
    delete[](vertices);
    delete[](normals);
}

bool DrawTrigs(QOpenGLFunctions *m_context)
{
    stl_file.DrawTriangles(m_context);
    return(stl_file.NumTrigs() > 0);
}

void importSTL(string &fn)
{
   stl_file.setFile(fn);
   stl_file.readSTL();
}

void StartExport()
{
    stl_file.ClearTrigs();
}

void EndExport(QString fn, std::vector<vet3> &xyzat)
{
    stl_file.setFile(fn.toStdString());
    stl_file.writeSTL(xyzat);
    stl_file.ClearTrigs();
}
